# -*- coding: utf-8 -*-
from json import dumps
from pydantic import BaseModel, ConfigDict, Field


class CommonResult(BaseModel):

    model_config = ConfigDict(arbitrary_types_allowed=True)

    code: int = Field(default=200, description="status code")

    message: str = Field(default="OK", description="response message")

    data: object = Field(default=None, description="response data")

    def __init__(self, code: int, message: str, data: object):
        """
        构造方法\n
        :param code: 状态码
        :param message: 消息
        :param data: 响应正文
        """
        super().__init__()
        self.code = code
        self.message = message
        self.data = data

    @staticmethod
    def ok(message: str = "OK", data: object = None):
        """
        请求成功\n
        :param message: 消息
        :param data: 响应正文
        :return: 响应
        """
        result = CommonResult(code=200, message=message, data=data)
        return result

    @staticmethod
    def bad_request(message: str = "Bad Request", data: object = None):
        """
        请求坏了\n
        :param message: 消息
        :param data: 响应正文
        :return: 响应
        """
        result = CommonResult(code=400, message=message, data=data)
        return result

    @staticmethod
    def not_found(message: str = "Not Found", data: object = None):
        """
        没找到\n
        :param message: 消息
        :param data: 响应正文
        :return: 响应
        """
        result = CommonResult(code=404, message=message, data=data)
        return result

    @staticmethod
    def failed(message: str = "Internal Server Error", data: object = None):
        """
        请求失败\n
        :param message: 消息
        :param data: 响应正文
        :return: 响应
        """
        result = CommonResult(code=500, message=message, data=data)
        return result

    @staticmethod
    def make_response(code: int, message: str, data: object):
        """
        制作一个响应\n
        :param code: 状态码
        :param message: 消息
        :param data: 响应正文
        :return: 响应
        """
        result = CommonResult(code=code, message=message, data=data)
        return result

    def __str__(self) -> str:
        return dumps(self.__dict__)


if __name__ == '__main__':
    print(CommonResult.failed("ll"))



# -*- coding: utf-8 -*-
from time import time
from types import FunctionType
from typing import Tuple, Any


def execute_time(func: FunctionType, *args, **kwargs) -> Tuple[Any, int]:
    """
    执行函数并统计用时多少毫秒\n
    :param func: 要执行的函数
    :param args: 可变长度参数
    :param kwargs: 关键字参数
    :return: (原函数返回结果, 执行毫秒数)
    """
    start_time = int(round(time() * 1000))
    result = func(*args, **kwargs)
    end_time = int(round(time() * 1000))
    return result, end_time - start_time



# -*- coding: utf-8 -*-
import inspect
from types import FunctionType
from functools import wraps
from flask import Flask, has_app_context, current_app
from flask_sqlalchemy import SQLAlchemy


def add_app_context(app: Flask, func: FunctionType) -> FunctionType:
    """
    给函数加上上下文\n
    :param app: Flask对象
    :param func: 函数
    :return: 有上下文的函数
    """
    @wraps(func)
    def result(*args, **kwargs):
        with app.app_context():
            return func(*args, **kwargs)
    return result


def get_sqlalchemy() -> SQLAlchemy:
    """
    获取SQLalchemy对象\n
    :return: SQLalchemy对象, 没找到返回空
    """
    if has_app_context():
        if "sqlalchemy" in current_app.extensions.keys():
            return current_app.extensions["sqlalchemy"]
        else:
            raise ValueError("未连接数据库, 未找到绑定Flask的数据源")
    else:
        main_module = __import__("__main__")
        for _name_, _var_ in inspect.getmembers(main_module, lambda x: isinstance(x, Flask)):
            if "sqlalchemy" in _var_.extensions.keys():
                return _var_.extensions["sqlalchemy"]
        raise ValueError("未连接数据库, 未找到绑定Flask的数据源")



# -*- coding: utf-8 -*-
import re
import json
from functools import wraps
from typing import Union, Collection, Type
from collections.abc import Iterable
import inspect
from urllib.parse import unquote
from enum import Enum
from flask import request, Response
from flask_ssm.utils.module_utils import blueprint_from_module


class RequestMethod(Enum):
    """
    请求方法\n
    """
    GET = "GET"
    HEAD = "HEAD"
    POST = "POST"
    PUT = "PUT"
    PATCH = "PATCH"
    DELETE = "DELETE"
    OPTIONS = "OPTIONS"
    TRACE = "TRACE"


class RequestMapping:
    """
    用它修饰请求接口函数\n
    """
    def __init__(self, value: str, method: Union[str, Collection[str], RequestMethod, Collection[RequestMethod]]):
        """
        构造方法\n
        :param value: 请求路径
        :param method: 请求方法
        """
        self.rule = re.sub(r'\{([^}]+)\}', r'<\1>', value)
        if type(method) is str:
            self.methods = [method]
        elif type(method) is RequestMethod:
            self.methods = [method.value]
        elif isinstance(method, Iterable):
            self.methods = list(map(lambda x: x if type(x) is str else x.value, method))
        else:
            raise ValueError(f"Unknown parameter method={method}")

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        @wraps(func)
        def result(*args, **kwargs):
            if request.is_json:
                json_value = request.get_json(silent=True)
                if json_value is not None:
                    kwargs.update(json_value)
            elif request.mimetype.startswith("application/x-www-form-urlencoded") or request.mimetype.startswith("multipart/form-data"):
                values = request.values
                values = dict(zip(values.keys(), map(lambda x: unquote(x), values.values())))
                kwargs.update(values)
                if request.mimetype.startswith("multipart/form-data"):
                    kwargs.update(request.files)
            kwargs.update(dict(zip(inspect.signature(func).parameters.keys(), args)))
            if "" in kwargs.keys():
                kwargs.pop("")
            _module_ = inspect.getmodule(func)
            _inner_result_ = func(**kwargs)
            if inspect.getmembers(_module_, lambda x: x is RestController):
                return Response(status=200, response=json.dumps(_inner_result_, default=lambda x: x.__dict__), content_type="application/json;charset=utf-8")
            else:
                return _inner_result_
        bp = blueprint_from_module(func)
        return bp.route(self.rule, methods=self.methods)(result)


class GetMapping(RequestMapping):
    """
    用它修饰GET请求接口函数\n
    """
    def __init__(self, value: str):
        """
        构造方法\n
        :param value: 请求路径
        """
        super().__init__(value, RequestMethod.GET)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        return super().__call__(func)


class PostMapping(RequestMapping):
    """
    用它修饰POST请求接口函数\n
    """
    def __init__(self, value: str):
        """
        构造方法\n
        :param value: 请求路径
        """
        super().__init__(value, RequestMethod.POST)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        return super().__call__(func)


class PutMapping(RequestMapping):
    """
    用它修饰PUT请求接口函数\n
    """
    def __init__(self, value: str):
        super().__init__(value, RequestMethod.PUT)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        return super().__call__(func)


class DeleteMapping(RequestMapping):
    """
    用它修饰DELETE请求接口函数\n
    """
    def __init__(self, value: str):
        super().__init__(value, RequestMethod.DELETE)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        return super().__call__(func)


class PatchMapping(RequestMapping):
    """
    用它修饰PATCH请求接口函数\n
    """
    def __init__(self, value: str):
        super().__init__(value, RequestMethod.PATCH)

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        return super().__call__(func)


class ExceptionHandler:
    """
    用来统一处理同一个模块的接口异常\n
    """
    def __init__(self, value: Type[Exception] = Exception):
        """
        构造方法\n
        :param value: 捕获的异常
        """
        self.exception_cls = value

    def __call__(self, func):
        """
        执行函数\n
        :param func: 原函数
        :return:
        """
        bp = blueprint_from_module(func)
        return bp.errorhandler(self.exception_cls)(func)


class ResponseBody:
    """
    把函数返回值的json写入响应体\n
    """
    def __init__(self, func):
        """
        构造函数\n
        :param func: 装饰的函数
        """
        wraps(func)(self)
        self.func = func

    def __call__(self, *args, **kwargs):
        """
        调用函数\n
        :param args: 变长参数
        :param kwargs: 关键字参数
        :return: 修改后的返回值
        """
        result = self.func(*args, **kwargs)
        if type(result) is Response:
            return result
        else:
            return Response(status=200, response=json.dumps(result, default=lambda x: x.__dict__), content_type="application/json;charset=utf-8")


class RestController:
    pass



# -*- coding: utf-8 -*-
import os
import sys
from typing import List
from types import ModuleType, MethodType
import inspect
import pkgutil
import logging
from pathlib import Path
from functools import wraps
from werkzeug.middleware.dispatcher import DispatcherMiddleware
from flask import Flask, Blueprint
from flask.config import Config
from flask_apscheduler import APScheduler
from flask_sqlalchemy import SQLAlchemy
from flask_cors import CORS
from flask_ssm.springframework.stereotype import Controller, Service, Repository
from flask_ssm.springframework.context.annotation import Configuration
from flask_ssm.springframework.scheduling.annotation import Scheduled
from flask_ssm.utils.module_utils import walk_sub_modules, run_with_outside_server
from flask_ssm.utils.context_utils import add_app_context


class SpringApplication:

    def __init__(self):
        """
        构造方法\n
        """
        # 记录下哪些包需要自动导入
        self.__config_packages__: List[ModuleType] = list()
        self.__controller_packages__: List[ModuleType] = list()
        self.__service_packages__: List[ModuleType] = list()
        self.__dao_packages__: List[ModuleType] = list()
        self.__task_packages__: List[ModuleType] = list()
        # 遍历子包，自动导入
        self.base_package = inspect.getmodule(inspect.stack()[1].frame)
        for _sub_package_ in pkgutil.iter_modules([os.path.dirname(self.base_package.__file__)]):
            if _sub_package_.ispkg:
                _import_package_ = __import__(".".join([self.base_package.__package__, _sub_package_.name]), fromlist=[_sub_package_.name])
                if inspect.getmembers(_import_package_, lambda x: x is Configuration):
                    self.__config_packages__.append(_import_package_)
                    logging.info("已发现config包: " + _import_package_.__package__)
                if inspect.getmembers(_import_package_, lambda x: x is Controller):
                    self.__controller_packages__.append(_import_package_)
                    logging.info("已发现controller包: " + _import_package_.__package__)
                if inspect.getmembers(_import_package_, lambda x: x is Repository):
                    self.__dao_packages__.append(_import_package_)
                    logging.info("已发现dao包: " + _import_package_.__package__)
                if inspect.getmembers(_import_package_, lambda x: x is Service):
                    self.__service_packages__.append(_import_package_)
                    logging.info("已发现service包: " + _import_package_.__package__)
                if inspect.getmembers(_import_package_, lambda x: x is Scheduled):
                    self.__task_packages__.append(_import_package_)
                    logging.info("已发现task包: " + _import_package_.__package__)

    def init_app(self, app: Flask):
        """
        初始化框架\n
        :param app: Flask对象
        :return:
        """
        self.__init_self__(app)
        self.__init_config__(app)
        self.__init_dao__(app)
        self.__init_service__(app)
        self.__init_controller__(app)
        self.__init_task__(app)
        self.__init_cors__(app)
        self.__init_run__(app)

    def __init_self__(self, app: Flask):
        """
        初始化Flask-SSM与Flask的关联\n
        """
        if not hasattr(app, 'extensions'):
            app.extensions = dict()
        app.extensions["spring"] = self
        app.jinja_env.globals['spring'] = self
        app.config.setdefault('APP_HOST', '0.0.0.0')
        app.config.setdefault('APP_PORT', 5000)
        app.config.setdefault('APP_THREAD', False)
        app.config.setdefault('APP_PROCESS', 1)
        app.config.setdefault("APP_STATIC", "static")
        app.config.setdefault("APP_TEMPLATES", "templates")
        app.config.setdefault("APP_RELOADER", False)
        app.config.setdefault("APPLICATION_ROOT", "/")
        app.config.setdefault("DEBUG", False)

    def __init_config__(self, app: Flask):
        """
        初始化配置包\n
        :param app: Flask对象
        :return:
        """
        for __config_package__ in self.__config_packages__:
            for __config_module__ in walk_sub_modules(__config_package__):
                app.config.from_object(__config_module__)
                for __key__ in dir(__config_module__):
                    if __key__.isupper():
                        __value__ = app.config.get(__key__)
                        __env_value__ = os.getenv(__key__)
                        if __env_value__ is not None:
                            logging.warning(f"使用环境变量{__key__}={__env_value__}配置项")
                            if type(__value__) is str:
                                app.config[__key__] = __env_value__
                            else:
                                app.config[__key__] = eval(__env_value__)
        if len(logging.root.handlers) <= 1:
            __import__("flask_ssm.logging.logs_config")
        app.logger.info("初始化配置成功")

    def __init_controller__(self, app: Flask):
        """
        初始化controller\n
        :param app: Flask对象
        :return:
        """
        for __controller_package__ in self.__controller_packages__:
            for __controller_module__ in walk_sub_modules(__controller_package__):
                _blueprints_ = inspect.getmembers(__controller_module__, lambda x: isinstance(x, Blueprint))
                for _var_name_, _blueprint_ in _blueprints_:
                    if "__blueprint__" == _var_name_:
                        _blueprint_.template_folder = app.config.get("APP_TEMPLATES")
                        _blueprint_.static_folder = app.config.get("APP_STATIC")
                        _blueprint_.root_path = os.path.dirname(os.path.abspath(sys.argv[0]))
                    app.register_blueprint(_blueprint_)
        app.logger.info("初始化视图成功")

    def __init_dao__(self, app: Flask):
        """
        初始化数据交互层\n
        :param app: Flask对象
        :return:
        """
        db = SQLAlchemy()
        db.init_app(app)
        app.logger.info("初始化数据库连接成功")

    def __init_task__(self, app: Flask):
        """
        初始化定时任务\n
        :param app: Flask对象
        :return:
        """
        scheduler = APScheduler()
        jobs = list()
        for __task_package__ in self.__task_packages__:
            for __task_module__ in walk_sub_modules(__task_package__):
                cfg = Config(os.path.dirname(os.path.abspath(Path(__task_package__.__file__))))
                cfg.from_object(__task_module__)
                if not cfg.get("FUNC", None):
                    continue
                else:
                    __func__ = cfg.get("FUNC")
                    cfg["FUNC"] = __task_module__.__name__ + ":" + cfg.get("FUNC")
                    _task_function_ = getattr(__task_module__, __func__)
                    setattr(__task_module__, __func__, add_app_context(app, _task_function_))
                if "ID" not in cfg.keys():
                    cfg["ID"] = __task_module__.__name__
                cfg = {k.lower(): v for k, v in cfg.items() if k.isupper()}
                jobs.append(dict(cfg))
        app.config.update({
            "JOBS": jobs
        })
        if self.__task_packages__ and scheduler:
            scheduler.init_app(app)
            scheduler.start()
            app.logger.info("初始化定时任务成功")

    def __init_service__(self, app: Flask):
        """
        初始化service包\n
        :param app: Flask对象
        :return:
        """
        app.logger.info("初始化业务逻辑成功")

    def __init_cors__(self, app: Flask):
        """
        初始化跨域\n
        :param app:
        :return:
        """
        CORS(app, supports_credentials=True)
        app.logger.info("初始化跨域成功")

    def __init_run__(self, app: Flask):
        """
        修改app的run函数\n
        :param app: Flask对象
        :return:
        """
        @wraps(app.run)
        def run(*args, **kwargs):
            kwargs.update(dict(zip(inspect.signature(app.run).parameters.keys(), args)))
            kwargs["host"] = kwargs.get("host", app.config.get("APP_HOST", None))
            kwargs["port"] = kwargs.get("port", app.config.get("APP_PORT", None))
            kwargs["threaded"] = kwargs.get("threaded", app.config.get("APP_THREAD", None))
            kwargs["processes"] = kwargs.get("processes", app.config.get("APP_PROCESS", None))
            kwargs["use_reloader"] = kwargs.get("use_reloader", app.config.get("APP_RELOADER", None))
            return Flask.run(app, **kwargs)
        mounts = None if app.config.get("APPLICATION_ROOT", "/") == "/" else {app.config.get("APPLICATION_ROOT", "/"): app}
        app.wsgi_app = DispatcherMiddleware(app.wsgi_app, mounts=mounts)
        app.run = run
        app.run_with_outside_server = MethodType(run_with_outside_server, app)
        app.logger.info("初始化运行入口成功")



# -*- coding: utf-8 -*-
import sys
import logging
import inspect
from inspect import Parameter
from typing import Type, Tuple
from types import FunctionType
from pydantic import create_model, ValidationError, TypeAdapter


if sys.version_info >= (3, 8):
    from typing import get_origin
else:
    from typing_inspect import get_origin


def __get_origin__(tp):
    """
    对系统get_origin的扩展\n
    :param tp: 输入类型
    :return: 返回get_origin的类型或原类型
    """
    _result_ = get_origin(tp)
    return tp if _result_ is None else _result_


def pojo_private_properties(cls: Type) -> dict:
    """
    获取pojo类的私有属性\n
    :param cls: pojo类
    :return: 属性字典
    """
    result = dict()
    if hasattr(cls, "__tablename__"):
        result["__tablename__"] = getattr(cls, "__tablename__")
    if hasattr(cls, "__table__"):
        result["__table__"] = getattr(cls, "__table__")
    if hasattr(cls, "__bind_key__"):
        result["__bind_key__"] = getattr(cls, "__bind_key__")
    if hasattr(cls, "__table_args__"):
        result["__table_args__ "] = getattr(cls, "__table_args__")
    return result


def validate_value_with_type(value_type, value):
    """
    验证value是否符合value_type\n
    :param value_type: 参数类型
    :param value: 数值
    :return: 验证或转换类型后的value
    """
    model = create_model('DynamicModel', value=(value_type, ...))
    validated_value = model(value=value)
    return validated_value.value


def validate_single_value(value_type, value):
    """
    验证某一个value的类型\n
    :param value_type: 参数类型
    :param value: 数值
    :return: 验证或转换类型后的value
    """
    try:
        return validate_value_with_type(value_type, value)
    except ValidationError as e:
        logging.warning(e)
        return value


def function_parameter_defaults(func):
    """
    过去函数参数默认值\n
    :param func: 函数
    :return: 默认值组成的字典
    """
    sig = inspect.signature(func)
    defaults = {}
    for name, param in sig.parameters.items():
        if param.default is not Parameter.empty:
            defaults[name] = param.default
    return defaults


def validate_params(func: FunctionType, kwargs: dict) -> Tuple[dict, dict]:
    """
    验证函数的参数是否符合类型\n
    :param func: 函数
    :param kwargs: 函数的实参
    :return: 已验证的, 未验证的
    """
    errors = dict()
    validated = dict()
    function_defaults = function_parameter_defaults(func)
    for _name_, _type_ in func.__annotations__.items():
        if "return" == _name_:
            continue
        _input_ = kwargs.get(_name_)
        try:
            adapter = TypeAdapter(_type_)
            validated[_name_] = adapter.validate_python(_input_)
        except ValidationError as e:
            if _input_ is None and _name_ in function_defaults.keys():
                try:
                    validated[_name_] = validate_value_with_type(_type_, function_defaults[_name_])
                except ValidationError as e:
                    err = e.errors()[0]
                    errors[_name_] = err
            else:
                err = e.errors()[0]
                errors[_name_] = err
    kwargs = {**kwargs, **validated}
    return kwargs, errors
